package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 21:44
 */
@Getter
@Setter
@SuperBuilder
public class TimePressureMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 8128429933652613769L;
    /**
     * 时压模式初始气井状态
     */
    @Builder.Default
    private boolean tpInitStatus = false;
    /**
     * 时压模式开井源
     */
    @Builder.Default
    private TimePressureOpenSouceEnum tpOpenSource = TimePressureOpenSouceEnum.OIL;
    /**
     * 时压模式开井压力触发模式：
     * 0：大于压力触发
     * 1：小于压力触发
     */
    @Builder.Default
    private int tpOpenTrigger = 0;
    /**
     * 时压模式开井压力，[0~9999]
     */
    @Builder.Default
    private int tpOpenPressure = 0;
    /**
     * 时压模式开井时长
     */
    @Builder.Default
    private String tpOpenTiming = "00:00:00";
    /**
     * 时压模式关井源
     */
    @Builder.Default
    private TimePressureOpenSouceEnum tpCloseSource = TimePressureOpenSouceEnum.OIL;
    /**
     * 时压模式关井触发
     * 0：大于压力触发
     * 1：小于压力触发
     */
    @Builder.Default
    private int tpCloseTrigger = 0;
    /**
     * 时压模式关井压力，[0~9999]
     */
    @Builder.Default
    private int tpClosePressure = 0;
    /**
     * 时压模式关井时长
     */
    @Builder.Default
    private String tpCloseTiming = "00:00:00";
    /**
     * 时压模式压力稳定时长：秒，[0-120]
     */
    @Builder.Default
    private int tpStabilityTime = 0;

    public TimePressureMode() {
        super(5);
        this.setCode(Command.KNPCV1_RUN_TIME_PRESSURE);
    }

    @Override
    public boolean validate() {
        return super.validate()
                && Objects.nonNull(this.tpOpenSource)
                && Objects.nonNull(this.tpCloseSource)
                && StringUtils.isNotBlank(this.tpOpenTiming)
                && StringUtils.isNotBlank(this.tpCloseTiming)
                && this.tpOpenTrigger >= 0 && this.tpOpenTrigger <= 1
                && this.tpCloseTrigger >= 0 && this.tpCloseTrigger <= 1
                && this.tpOpenPressure >= 0 && this.tpOpenPressure <= 9999
                && this.tpClosePressure >= 0 && this.tpClosePressure <= 9999
                && this.tpStabilityTime >= 0 && this.tpStabilityTime <= 120;
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(150);
        //                                 000E*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     0177     000E     1C
        command.append("01100177000E1C");
        command.append("000").append(this.tpInitStatus ? "1" : "0");
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpOpenSource.getCode()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpOpenTrigger), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpOpenPressure), 4, "0"));
        command.append(this.toHexString(this.tpOpenTiming));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpCloseSource.getCode()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpCloseTrigger), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpClosePressure), 4, "0"));
        command.append(this.toHexString(this.tpCloseTiming));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tpStabilityTime), 4, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setTpOpenTiming(String tpOpenTiming) {
        this.tpOpenTiming = this.timingValidate(tpOpenTiming, "tpOpenTiming");
    }

    public void setTpCloseTiming(String tpCloseTiming) {
        this.tpCloseTiming = this.timingValidate(tpCloseTiming, "tpCloseTiming");
    }

    /**
     * 时压模式开井源：
     * 0：油压
     * 1：套压
     * 2：输压
     * 3：差压
     * <p>
     * 4：时间
     */
    @Getter
    public enum TimePressureOpenSouceEnum {
        OIL(0, "油压"),
        CAS(1, "套压"),
        PRE(2, "输压"),
        DIFF(3, "差压"),
        TIME(4, "时间");

        private final int code;
        private final String name;

        TimePressureOpenSouceEnum(int code, String name) {
            this.code = code;
            this.name = name;
        }
    }

}
